using System;
using cAlgo.API;
using cAlgo.API.Indicators;
using System.Collections.Generic;

namespace cAlgo.Robots
{
    partial class History: IComparable<History>
    	{
    		public Position pos;
    		public DateTime otime;
    		public int CompareTo(History other)
    		{
    			if(otime>other.otime) return 1;
    			if(otime<other.otime) return -1;
    			return 0;
    		}
    	}
    	
    [Robot]
    public class ComFracti : Robot
    {
        
        [Parameter("tp1",DefaultValue=2500)]
        public double tp1 { get; set; }
        
        [Parameter("sl1",DefaultValue=500)]
        public double sl1 { get; set; }
        
        [Parameter("Expimin",DefaultValue=5555)]
        public int Expimin { get; set; }
        
        [Parameter("Risk",DefaultValue=0.31)]
        public double Risk { get; set; }
        
        [Parameter("multilot",DefaultValue=1)]
        public int multilot { get; set; }
        
        [Parameter("closeby",DefaultValue=0)]
        public int closebyint { get; set; }
        
        [Parameter("RSI",DefaultValue=18)]
        public int RSI { get; set; }
        
        [Parameter("sh2",DefaultValue=52)]
        public int sh2 { get; set; }
        
        [Parameter("sh3",DefaultValue=25)]
        public int sh3 { get; set; }
        
        [Parameter("sh4",DefaultValue=68)]
        public int sh4 { get; set; }
        
        [Parameter("sh5",DefaultValue=53)]
        public int sh5 { get; set; }
        
        [Parameter("levelb",DefaultValue=3)]
        public int levelb { get; set; }
        
        [Parameter("levels",DefaultValue=3)]
        public int levels { get; set; }
        
        private bool closeby;
        private RelativeStrengthIndex rsi;
        private List<Position> pos = new List<Position>();
        private List<DateTime> oot= new List<DateTime>(); 
        private List<History> his= new List<History>(); 
        
        protected override void OnStart()
        {
            rsi=Indicators.RelativeStrengthIndex(MarketSeries.Open,RSI);
           	if(closebyint==0) closeby=false;
           	else
           		closeby=true;
        }

        protected override void OnTick()
        {
           int last=MarketSeries.Open.Count-1;
           if(!(MarketSeries.Open[last]==MarketSeries.High[last] && MarketSeries.Open[last]==MarketSeries.Low[last])) return;
           for(int i=0; i<pos.Count;i++)
           {
           		if(Server.Time.ToBinary()-oot[i].ToBinary()>=Expimin*60)
           		{
	           		Trade.Close(pos[i]);
	           		
	           	}
	           	if(pos[i].TradeType== TradeType.Buy && psv()  < 0 && closeby )
	           	{
	           		Trade.Close(pos[i]);
	           		
	           	}
	           	if(pos[i].TradeType== TradeType.Buy && psv()  > 0 && closeby)
	           	{
	           		Trade.Close(pos[i]);
	           		
	           	}
           }
           if(Trade.IsExecuting) return;
           if (psv()  > 0) 
           {
           		Trade.CreateBuyMarketOrder(Symbol,LotSize());
	   		}
	   		if (psv()  < 0)
	   		{
	   			Trade.CreateSellMarketOrder(Symbol,LotSize());
	   		}
        }

        protected override void OnStop()
        {
            // Put your deinitialization logic here
        }
        
        protected override void OnPositionOpened(Position openedPosition)
        {
        	pos.Add(openedPosition);
        	oot.Add(Server.Time);
        	if(openedPosition.TradeType== TradeType.Buy)
        		Trade.ModifyPosition(openedPosition,Symbol.Bid - sl1 * Symbol.PointSize, Symbol.Bid + tp1 * Symbol.PointSize);
        	if(openedPosition.TradeType== TradeType.Sell)
        		Trade.ModifyPosition(openedPosition,Symbol.Ask+sl1*Symbol.PointSize,Symbol.Ask-sl1*Symbol.PointSize);
        }
        
        protected override void OnPositionClosed(Position position)
        {
        	for(int i=0; i<pos.Count;i++)
        	{
        		if(pos[i].Id==position.Id)
				{
					History hs=new History();
					hs.pos=pos[i];
					hs.otime=oot[i];
					his.Add(hs);
					his.Sort();
					pos.RemoveAt(i);
	           		oot.RemoveAt(i);
	           		break;
				}				
        	}
        }
        
        private double Fractalup(int index)
        {
        	index=MarketSeries.Open.Count-1-index;
        	if(index<2) return 0.0;
        	if(MarketSeries.High[index]>MarketSeries.High[index+1] && MarketSeries.High[index]>MarketSeries.High[index+2] && MarketSeries.High[index]>MarketSeries.High[index-1] && MarketSeries.High[index]>MarketSeries.High[index-2])
            	return MarketSeries.High[index];
            return 0;
        }
        
        private double Fractaldw(int index)
        {
        	index=MarketSeries.Open.Count-1-index;
        	if(index<2) return 0.0;
        	if(MarketSeries.Low[index]<MarketSeries.Low[index+1] && MarketSeries.Low[index]<MarketSeries.Low[index+2] && MarketSeries.Low[index]<MarketSeries.Low[index-1] && MarketSeries.Low[index]<MarketSeries.Low[index-2])
            	return MarketSeries.Low[index];
            return 0;
        }
        private int LotSize()
        {
        	int DecreaseFactor=multilot;
  
   			
   			int    orders=his.Count;     
  			int    losses=0;                  
			
			int lot=(int)(Account.FreeMargin*Risk/1000)*10000;
			if(lot<10000) lot=10000;
			
			if(DecreaseFactor>0)
     		{
     			for(int i=orders-1;i>=0;i--)
     			{
     				if(his[i].pos.Profit>0) break;
     				if(his[i].pos.Profit<0) losses++;
     			}
     			if(losses>0) lot=lot+lot*DecreaseFactor;
     		}
     		return lot;
        }
        
        private double Crof( int s)    
   		{
     
  			double frup = Fractalup(s);
   			double frdw = Fractaldw(s);
  
    
    		if ( (frup==0 ) && frdw!=0 ) return (1); 
    		if ( (frdw==0 ) && frup!=0 ) return (-1); 
   
    
     
    		 return (0); 
   		} 
		
		private int psv()  
		{

 
 			double m6=rsi.Result[0];

   			if ( Crof( sh2)>0  && Crof( sh3)>0  && m6<50-levelb ) return(1);
    
   			if ( Crof( sh4)<0  && Crof( sh5)<0  && m6>50+levels ) return(-1);
      
      
   			return(0);
  
		}
  		
 
    }
    
}